using System;
using System.Net;
using System.Net.Sockets;
using System.Collections;

using NetSystem;
using NetSystem.Server;

using ChatLibrary;

namespace ChatServer
{
	class ChatServer
	{
		private NetServer m_Server;
		private Hashtable m_Users;

		public ChatServer(int thePort)
		{
			m_Users = new Hashtable();

			m_Server = new NetServer(thePort);
			m_Server.EventReceivedData			+= new ReceivedData(m_Server_EventReceivedData);
			m_Server.EventNewConnection			+= new NewConnection(m_Server_EventNewConnection);
			m_Server.EventClientDisconnected	+= new ClientDisconnected(m_Server_EventClientDisconnected);
			
			try
			{
				Console.WriteLine("Start listening on port {0}...", thePort);
				m_Server.StartListen();
			}
			catch(SocketException ex)
			{
				Console.WriteLine("Error: {0}", ex.Message);
				Console.ReadLine();
			}
		}

		#region Event Handler
		private void m_Server_EventReceivedData(int theClientID, NetPacket thePacket)
		{
			if(thePacket is NetLogin)
			{
				NetLogin msg = (NetLogin) thePacket;

				if(m_Users.ContainsValue(msg.Username))
				{
					m_Server.ClientDisconnect(theClientID, "Username already in use");
				}
				else
				{
					Console.WriteLine("User {0} logged in", msg.Username);

					string[] users = new string[m_Users.Count];
					m_Users.Values.CopyTo(users, 0);
					NetUserList userList = new NetUserList(users);
					m_Server.SendData(theClientID, userList);

					m_Users.Add(theClientID, msg.Username);
					sendToAll(theClientID, new NetUserJoin(msg.Username));
				}
			}
			else if(thePacket is NetMessage)
			{
				NetMessage msg = (NetMessage) thePacket;
				Console.WriteLine("Message from {0} : {1}", m_Users[theClientID], msg.Message);

				sendToAll(theClientID, new NetChatMessage(m_Users[theClientID].ToString(), msg.Message));
			}
		}

		private void m_Server_EventNewConnection(int theClientID, IPAddress theClientIP)
		{
			Console.WriteLine("New Connection from {0}. Id: {1}", theClientIP.ToString(), theClientID);
		}

		private void m_Server_EventClientDisconnected(int theClientID, string theReason)
		{
			Console.WriteLine("Client {0} disconnected. Reason: {1}", theClientID, theReason);

			sendToAll(theClientID, new NetUserLeft(m_Users[theClientID].ToString(), theReason));
			m_Users.Remove(theClientID);
		}
		#endregion

		public void Close()
		{
			foreach(int id in m_Users.Keys)
			{
				m_Server.ClientDisconnect(id, "Server shutdown");
			}

			m_Server.StopListen();
			m_Server.Close();
		}

		#region Private Methods
		private void sendToAll(int theClientID, NetPacket thePacket)
		{
			foreach(int id in m_Users.Keys)
			{
				if(id != theClientID)
				{
					m_Server.SendData(id, thePacket);
				}
			}
		}
		#endregion
	}
}