using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Net.Sockets;
using System.Runtime.Serialization.Formatters.Binary;

namespace NetSystem.Client
{
	public delegate void ReceivedDataFromServer(NetPacket thePacket);
	public delegate void DisconnectedFromServer(string theReason);

	public class NetClient
	{
		private Socket m_Socket;
		private IPEndPoint m_EndPoint;
		private NetworkStream m_NetStream;
		private BinaryFormatter m_BF;
		private Thread m_GetDataThread;

		public event ReceivedDataFromServer EventReceivedData;
		public event DisconnectedFromServer EventDisconnected;

		public NetClient()
		{
			m_BF		= new BinaryFormatter();
		}

		#region Public Methods
		public void Connect(IPAddress theServer, int thePort)
		{
			if(m_Socket == null || !m_Socket.Connected)
			{
				m_GetDataThread = new Thread(new ThreadStart(getData));
				m_GetDataThread.Name = "Get Data from Client Thread";

				m_EndPoint	= new IPEndPoint(theServer, thePort);
				m_Socket	= new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

				try
				{
					m_Socket.Connect(m_EndPoint);
				}
				catch(SocketException ex)
				{
					throw ex;
				}

				m_NetStream = new NetworkStream(m_Socket, FileAccess.ReadWrite, true);

				m_GetDataThread.Start();
			}
			else
			{
				throw new SocketException(10056);
			}
		}

		public void Disconnect(string theMessage)
		{
			if(m_Socket.Connected)
			{
				SendData(new NetDisconnect(theMessage));
				disconnect();
			}
		}

		public void Close()
		{
			if(m_GetDataThread.IsAlive & m_GetDataThread != null)
			{
				m_GetDataThread.Abort();
			}

			m_NetStream.Close();
			m_Socket.Close();
		}

		public void SendData(NetPacket thePacket)
		{
			if(Connected)
			{
				try
				{
					m_BF.Serialize(m_NetStream, thePacket);
				}
				catch(Exception ex)
				{
					if(ex.InnerException is SocketException)
					{
						throw ex.InnerException;
					}
					else
					{
						throw ex;
					}
				}
			}
			else
			{
				throw new SocketException(10057);
			}
		}
		#endregion

		#region Private Methods
		private void disconnect()
		{
			m_GetDataThread.Abort();
			m_GetDataThread = null;
			m_Socket = null;
		}

		private void getData()
		{
			while(m_Socket.Connected)
			{
				try
				{
					NetPacket packet = (NetPacket)m_BF.Deserialize(m_NetStream);
					
					if(packet is NetDisconnect)
					{
						EventDisconnected(((NetDisconnect)packet).Reason);
						disconnect();
						return;
					}
					else
					{
						EventReceivedData(packet);
					}
				}
				catch(ThreadAbortException)
				{
					m_Socket = null;
					return;
				}
				catch(Exception ex)
				{
					if(ex.InnerException is SocketException)
					{
						EventDisconnected(ex.InnerException.Message);
					}
					else
					{
						EventDisconnected(ex.Message);
					}

					disconnect();
					return;
				}
			}
		}
		#endregion
		
		public bool Connected
		{
			get { return m_Socket == null ? false : m_Socket.Connected; }
		}
	}
}