using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Net.Sockets;
using System.Runtime.Serialization.Formatters.Binary;

namespace NetSystem.Server
{
	internal class Client : IComparable
	{
		public delegate void ClientDisconnect(Client theClient, string theReason);
		public event ClientDisconnect EventDisconnect;
		public event ReceivedDataFromClient EventReceivedData;

		private int m_ID;
		private Socket m_Socket;
		private Thread m_GetDataThread;
		private NetworkStream m_NetStream;
		private BinaryFormatter m_BF;

		public Client(Socket theSocket, int theID)
		{
			m_Socket = theSocket;
			m_ID = theID;

			m_BF = new BinaryFormatter();
			m_NetStream = new NetworkStream(m_Socket, FileAccess.ReadWrite, true);
			
			m_GetDataThread = new Thread(new ThreadStart(getData));
			m_GetDataThread.Name = "Get Data from Client Thread";
			m_GetDataThread.Priority = ThreadPriority.Lowest;
			m_GetDataThread.IsBackground = true;
		}

		#region Public Methods
		public void StartReading()
		{
			m_GetDataThread.Start();
		}

		public void SendData(NetPacket thePacket)
		{
			try
			{
				m_BF.Serialize(m_NetStream, thePacket);
			}
			catch(Exception ex)
			{
				throw ex;
			}
		}

		public void Close()
		{
			if(m_GetDataThread.IsAlive)
			{
				m_GetDataThread.Abort();
			}
		}

		public int CompareTo(object obj)
		{
			if(m_ID < ((Client)obj).ClientID)
			{
				return -1;
			}
			else if(((Client)obj).ClientID == m_ID)
			{
				return 0;
			}
			else
			{
				return 1;
			}
		}
		#endregion

		#region Private Methods
		private void disconnect()
		{
			m_NetStream.Close();
			m_Socket.Close();
		}

		private void getData()
		{
			while(m_Socket.Connected)
			{
				try
				{
					NetPacket packet = (NetPacket)m_BF.Deserialize(m_NetStream);
					
					if(packet is NetDisconnect)
					{
						EventDisconnect(this, ((NetDisconnect)packet).Reason);
					}
					else
					{
						EventReceivedData(packet, this);
					}
				}
				catch(ThreadAbortException)
				{
					disconnect();
					Thread.ResetAbort();
					return;
				}
				catch(Exception ex)
				{
					EventDisconnect(this, ex.Message);
					disconnect();
					return;
				}
			}
		}
		#endregion

		public IPAddress ClientIP
		{
			get { return ((IPEndPoint)m_Socket.RemoteEndPoint).Address; }
		}

		public int ClientID
		{
			get { return m_ID; }
		}
	}
}